import os
import time

from tqdm import tqdm

from Simbiber.BibTool import BibTool


class SimBiberTool():
    def __init__(self, args):
        self.args = args

    def init(self):
        with open(self.args.output_path, 'w', encoding='utf-8') as bibfile:
            bibfile.write('')

    def __simplify_and_write__(self,s):
        self.bib.simplify_bib(s)


    def __rewrite__(self):
        with open(self.args.output_path, 'r', encoding='utf-8') as file1:
            with open(self.args.input_path, 'w', encoding='utf-8') as file2:
                    file2.writelines(file1.readlines())

        os.remove(self.args.output_path)


    def __simplify__(self):
        if not self.args.if_append_output and not self.use_temp_file:
            self.init()
        with open(self.args.input_path, encoding='utf-8') as bibtex_file:
            l = []
            s = ''
            index = 0
            for line in tqdm(bibtex_file.readlines()):
                # ignore useless line
                if len(l) == 0 and line[0] != '@':
                    continue
                for i, x in enumerate(line):
                    if x == '{':
                        l.append('{')
                    elif x == '}':
                        l.pop()
                s += line
                if len(l) == 0:
                    index += 1
                if len(l) == 0 and index == self.args.cache_num:
                    self.__simplify_and_write__(s)
                    s = ''
                    index = 0
            if index != 0:
                self.__simplify_and_write__(s)
        self.bib.remove_duplication()
        self.bib.write_to_file()
        if self.use_temp_file:
            self.__rewrite__()

    def __judge_use_temp__(self):
        self.use_temp_file = False
        if self.args.output_path == '' or self.args.output_path == self.args.input_path:
            self.use_temp_file = True
            self.args.output_path = self.args.input_path + time.strftime("%Y%m%d%H%M%S", time.localtime())

    def simplify(self):
        input_path=self.args.input_path
        if os.path.isdir(input_path):
            for root, ds, fs in os.walk(input_path):
                for f in fs:
                    self.args.input_path=os.path.join(root, f)
                    self.args.output_path = os.path.join('out', f)
                    self.bib = BibTool(self.args)
                    self.__judge_use_temp__()
                    self.__simplify__()
        else:
            self.bib = BibTool(self.args)
            self.__judge_use_temp__()
            self.__simplify__()



import sys
import time

import bibtexparser
from bibtexparser.bparser import BibTexParser
from bibtexparser.bwriter import BibTexWriter
import json
import re
import os
class BibTool:
    # Assume that bib_string has valid structure
    def __init__(self, args):
        self.args = args
        self.dictionary={}
        self.bib_database = None
        self.index=0
        self.pattern_list,self.pattern_dict = self.read_pattern_config()

    def get_bib_text(self):
        return bibtexparser.dumps(self.bib_database)

    def read_pattern_config(self):
        pattern_dict={}
        if os.path.isdir(self.args.config_path):
            for root, ds, fs in os.walk(self.args.config_path):
                for f in fs:
                    with open(root + "/" + f.strip(), "r", encoding='utf-8') as load_f:
                        pattern_dict.update(json.load(load_f))
        else:
            with open(self.args.config_path, "r", encoding='utf-8') as load_f:
                pattern_dict = json.loads(load_f)

        return sorted(pattern_dict.keys(), key=len,reverse=True),pattern_dict

    def __simplify_bib__(self,item):
        if 'archiveprefix' in item:
            item['archivePrefix'] = item['archiveprefix']
            del item['archiveprefix']
            if 'primaryclass' in item:
                item['primaryClass'] = item['primaryclass']
                del item['primaryclass']
            return item
        if 'author' not in item:
            return item
        temp_item = {'ENTRYTYPE': item['ENTRYTYPE'],
                     'ID': item['ID'],
                     'author': item['author'],
                     'title': item['title'], }
        if 'year' in item:
            temp_item['year'] = item['year']
        if item['ENTRYTYPE'] == 'book':
            if 'address' in item:
                temp_item['address'] = item['address']
            if 'publisher' in item:
                temp_item['publisher'] = item['publisher']
            return temp_item
        if 'booktitle' in item:
            booktitle = item['booktitle'].replace('\n', ' ').replace('\&', 'and')
            booktitle = booktitle.replace('{', '').replace('}', '').replace('  ', ' ').replace('[', '').replace(']', '')
            for key in self.pattern_list:
                m = re.search(key.lower(), booktitle.lower())
                if m is not None:
                    booktitle = 'Proc. of ' + self.pattern_dict[key]
                    break
            temp_item['booktitle'] = booktitle

        if 'journal' in item:
            journal=item['journal']
            temp = journal.replace('\n', ' ')
            temp = temp.replace('{', '').replace('}', '').replace('  ', ' ').replace('[', '').replace(']', '')
            if temp.lower()=='advances in neural information processing systems':
                journal='Proc. of NeurIPS'
            else:
                m = re.search('AAAI', journal)
                if m is not None:
                    journal = 'Proc. of AAAI'

            temp_item['journal'] = journal
            return temp_item
        return temp_item

    def mark_duplicate(self,item):
        if item['title'].lower() in self.dictionary:
            temp_index = self.index
            if 'booktitle' in item or ('journal' in item and not item['journal'].lower().find('arxiv')>=0):
                temp_index = self.dictionary[item['title'].lower()]
            self.bib_database.entries[temp_index] = '#'
        else:
            self.dictionary[item['title'].lower()] = self.index

    def simplify_bib(self,bib_string):
        parser = BibTexParser(common_strings=True)
        parser.ignore_nonstandard_types = True

        if self.bib_database is None:
            self.bib_database = bibtexparser.loads(bib_string, parser=parser)
        else:
            self.bib_database.entries += bibtexparser.loads(bib_string, parser=parser).entries

        while self.index < len(self.bib_database.entries):
            item=self.bib_database.entries[self.index]
            if item!='#':
                self.bib_database.entries[self.index]=self.__simplify_bib__(item)
                if self.args.remove_duplicate:
                    self.mark_duplicate(item)
            self.index += 1

    def remove_duplication(self):
        if self.args.remove_duplicate:
            self.bib_database.entries=[x for x in self.bib_database.entries if x != '#']

    def write_to_file(self):
        writer = BibTexWriter()
        print("Writing...")
        with open(self.args.output_path, 'a', encoding='utf-8') as bibfile:
            bibfile.write(writer.write(self.bib_database))
        print("Finished.")


#!python
# -*- coding:utf-8 -*-
from __future__ import print_function
from setuptools import setup, find_packages
# import Simbiber

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

setup(
    name="simbiber",
    version="0.6.1",
    author="Qiguang Chen",
    author_email="charleschen2333@gmail.com",
    description="a tool to fix and simplify bib automatically.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    url="https://github.com/MLNLP-World/SimBiber",
    packages=find_packages('Simbiber',exclude=['test']),
    include_package_data = True,
    install_requires=[
        "argparse",
        "bibtexparser>=1.2.0",
        "tqdm"
        ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        'Topic :: Text Editors :: Text Processing',
        'Topic :: Text Processing :: Filters',
        "Topic :: Text Processing :: Markup :: LaTeX",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        'Programming Language :: Python :: Implementation :: CPython',
    ],
    entry_points={
        'console_scripts': ['simbiber = Simbiber.main:main'],
    },
    package_data={
        "Simbiber": ["config/*.json","parserConfig.json","data/*.bib","out/*.bib"],
    },
    py_modules=['Simbiber.main', 'Simbiber.SimBiberTool', 'Simbiber.BibTool'],
)



from argparse import ArgumentParser
import Simbiber
import sys
import os

from Simbiber.SimBiberTool import SimBiberTool
filepath = os.path.dirname(os.path.abspath(__file__)) + '/'

def main():
    def str2bool(v):
        if v.lower() in ('yes', 'true', 't', 'y', '1'):
            return True
        elif v.lower() in ('no', 'false', 'f', 'n', '0'):
            return False
        else:
            return False

    parser = ArgumentParser()
    parser.add_argument("-i", '--input_path', type=str, default=filepath+"data/bibtex.bib")
    parser.add_argument("-o", '--output_path', type=str, default=filepath+"out/bibtex.bib")
    parser.add_argument("-c", '--config_path', type=str, default=filepath+"config")
    parser.add_argument("-a",'--if_append_output', type=str2bool, default='False')
    parser.add_argument("-cch", '--cache_num', type=int, default=100)
    parser.add_argument("-r", '--remove_duplicate', type=str2bool, default='False')
    args = parser.parse_args()
    sim_biber = SimBiberTool(args)
    sim_biber.simplify()


if __name__ == '__main__':
    main()